'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _seleniumWebdriver = require('selenium-webdriver');

var _seleniumWebdriver2 = _interopRequireDefault(_seleniumWebdriver);

var _lodash = require('lodash');

var _DriverUtilities = require('./DriverUtilities');

var _SimpleLogger = require('./SimpleLogger');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var Eyes = require('eyes.selenium').Eyes;

// What config should look like at least:
var ConfigStructure = {
  defaultTimeout: 10000,
  defaultPageLoadTimeout: 10000,
  applicationName: "used for logging",
  baseUrl: "http://somesite.com",
  seleniumGrid: {
    host: "",
    username: "",
    accessKey: ""
  },
  eyesConfig: {
    apiKey: ""
  },
  capabilities: {// specific things that need to be provided to BrowserStack/SauceLabs/SeGrid/etc.
    // Other properties that help decide what browser to use like browserName, platform, version, etc.
    // Also the API key usually needs to be in here somewhere (check specific documentation for cloud provider).
  },
  scrollBuffer: 25 };

var DriverContainer = function () {
  _createClass(DriverContainer, [{
    key: 'initialize',
    value: function initialize(local) {
      this.by = _seleniumWebdriver2.default.By;
      this.eyesOpened = false;

      var builder = new _seleniumWebdriver2.default.Builder();
      // http://seleniumhq.github.io/selenium/docs/api/javascript/module/selenium-webdriver/index_exports_Builder.html
      if (local) {
        this.driver = builder.forBrowser(this.config.capabilities.browserName).build();
      } else {
        this.driver = builder.usingServer(this.config.seleniumGrid.host).withCapabilities(this.config.capabilities).build();
      }

      var _this = this;

      return this.driver.getSession().then(function (sessionId) {
        _this.driver.allPassed = true;
        _this.driver.sessionID = sessionId.id_;
        _this.driver.manage().timeouts().implicitlyWait(_this.config.defaultTimeout);
        _this.driver.manage().timeouts().pageLoadTimeout(_this.config.defaultPageLoadTimeout);
        _this.driverUtilities = new _DriverUtilities.DriverUtilities(_this.config, _this.logger, _this.driver, _this.defaultTimeout);
      });
    }
  }, {
    key: 'openEyes',
    value: function openEyes(TestName, batchName, batchId) {
      // set up applitools eyes
      var _this = this;
      _this.eyes = new Eyes();
      _this.eyes.setApiKey(this.config.eyesConfig.apiKey);
      _this.eyes.setForceFullPageScreenshot(true);
      _this.eyes.setStitchMode(Eyes.StitchMode.CSS);
      _this.eyes.setBatch(batchName, batchId);
      _this.eyes.setDefaultMatchTimeout(3000); // TODO refine this stuff. Do we need to go more/less for the carosels? Can we provide this after opening eyes?

      return _this.eyes.open(_this.driver, _this.config.applicationName, TestName).then(function (driver) {
        _this.driver = driver;
        _this.eyesOpened = true;
      });
    }
  }, {
    key: 'closeEyes',
    value: function closeEyes() {
      var _this = this;
      return _this.quitBrowser().then(function () {
        return _this.eyes.close(false);
      }).then(function () {
        _this.eyesOpened = false;
      });
    }
  }, {
    key: 'forceCloseEyes',
    value: function forceCloseEyes() {
      if (this.eyesOpened) {
        this.eyes.abortIfNotClosed();
      }
    }
  }]);

  function DriverContainer(config, logger) {
    _classCallCheck(this, DriverContainer);

    this.logger = logger;
    if (!this.logger) {
      this.logger = new _SimpleLogger.SimpleLogger();
    }

    this.config = config;

    // set run name
    this.testName = this.config.applicationName;
    this.logger.runName = this.testName;
    if (!this.config.capabilities.name) this.config.capabilities.name = this.testName;
  }

  _createClass(DriverContainer, [{
    key: 'clearCookiesAndDeleteLocalStorage',
    value: function clearCookiesAndDeleteLocalStorage() {
      return this.clearCookies().then(function () {
        return this.deleteLocalStorage();
      });
    }
  }, {
    key: 'clearCookies',
    value: function clearCookies() {
      return this.driver.manage().deleteAllCookies();
    }
  }, {
    key: 'deleteLocalStorage',
    value: function deleteLocalStorage() {
      var _this2 = this;

      return this.driver.getCurrentUrl().then(function (url) {
        if (url.startsWith('data:') === false && url !== 'about:blank') {
          return _this2.driver.executeScript('window.localStorage.clear();');
        }
      });
    }
  }, {
    key: 'quitBrowser',
    value: function quitBrowser() {
      return this.driver.quit();
    }

    // TODO From the WP-e2e-tests, not sure if this can easily be cleaner, or modular to dismiss all alerts instead of default 3, maybe accept -1 as all?

  }, {
    key: 'dismissAlerts',
    value: function dismissAlerts() {
      var _this3 = this;

      var qty = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 3;

      (0, _lodash.times)(qty, function () {
        return _this3.driver.get('data:,').then(function () {}, function (err) {
          return _this3.driver.sleep(2000).then(function () {
            _this3.logger.warn('Accepting an open alert: \'' + err + '\'');
            return _this3.driver.switchTo().alert().then(function (alert) {
              return alert.accept();
            }, function () {});
          });
        });
      });
    }
  }]);

  return DriverContainer;
}();

exports.default = DriverContainer;