'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createFile = createFile;
exports.createFileWithFilename = createFileWithFilename;
exports.deleteFile = deleteFile;
exports.writeScreenshot = writeScreenshot;
exports.writeTextLogFile = writeTextLogFile;

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _seleniumWebdriver = require('selenium-webdriver');

var _seleniumWebdriver2 = _interopRequireDefault(_seleniumWebdriver);

var _sanitizeFilename = require('sanitize-filename');

var _sanitizeFilename2 = _interopRequireDefault(_sanitizeFilename);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// https://github.com/Automattic/wp-e2e-tests/blob/master/lib/media-helper.js

function createFile(notRandom) {
  var d = _seleniumWebdriver2.default.promise.defer();
  var randomImageNumber = Math.floor(Math.random() * 2);

  // If notRandom is set, always choose image 0 for visual comparison
  if (notRandom) {
    randomImageNumber = 0;
  }

  var originalImageName = 'image' + randomImageNumber;
  var originalFileName = originalImageName + '.jpg';
  var originalFile = _path2.default.resolve(__dirname, '../image-uploads/' + originalFileName);
  var newImageName = ('0000' + (Math.random() * Math.pow(36, 4) << 0).toString(36)).slice(-4); // random number
  var newFileName = newImageName + '.jpg';
  var newFile = _path2.default.resolve(__dirname, '../image-uploads/' + newFileName);
  _fsExtra2.default.copySync(originalFile, newFile);

  d.fulfill({
    imageName: newImageName,
    fileName: newFileName,
    file: newFile
  });

  return d.promise;
}

function createFileWithFilename(filename, skipNameCheck) {
  if (!skipNameCheck && (0, _sanitizeFilename2.default)(filename) !== filename) {
    throw new Error('Invalid filename given ' + filename);
  }

  var d = _seleniumWebdriver2.default.promise.defer();
  var originalFileName = 'image0.jpg';
  var originalFile = _path2.default.resolve(__dirname, '../image-uploads/' + originalFileName);
  var newFile = _path2.default.resolve(__dirname, '../image-uploads/' + filename);
  _fsExtra2.default.copySync(originalFile, newFile);

  d.fulfill({
    imageName: filename,
    fileName: filename,
    file: newFile
  });

  return d.promise;
}

function deleteFile(fileDetails) {
  var d = _seleniumWebdriver2.default.promise.defer();
  _fsExtra2.default.deleteSync(fileDetails.file);
  d.fulfill(true);
  return d.promise;
}

function writeScreenshot(data, prefix, i18nScreenshot) {
  if (prefix === undefined) {
    prefix = '';
  }
  var directoryName = '../screenshots';
  if (i18nScreenshot === true) {
    directoryName = '../screenshots-i18n';
  }
  var screenShotDir = _path2.default.resolve(__dirname, directoryName);
  if (!_fsExtra2.default.existsSync(screenShotDir)) {
    _fsExtra2.default.mkdirSync(screenShotDir);
  }
  var dateString = new Date().getTime().toString();
  var fileName = dateString + '-' + prefix + '.png';
  var screenshotPath = screenShotDir + '/' + fileName;
  _fsExtra2.default.writeFileSync(screenshotPath, data, 'base64');

  return screenshotPath;
}

function writeTextLogFile(textContent, prefix, pathOverride) {
  if (prefix === undefined) {
    prefix = '';
  }
  var directoryName = pathOverride || '../logs';
  var logDir = _path2.default.resolve(__dirname, directoryName);
  if (!_fsExtra2.default.existsSync(logDir)) {
    _fsExtra2.default.mkdirSync(logDir);
  }
  var dateString = new Date().getTime().toString();
  var fileName = dateString + '-' + prefix + '-log.txt';
  var logPath = logDir + '/' + fileName;
  _fsExtra2.default.writeFileSync(logPath, textContent);

  return logPath;
}