'use strict';

import * as MediaUtilities from './MediaUtilities';

export default class BaseComponent {
  constructor(driverContainer, friendlyName, expectedElementSelector, url) {
    this.driverContainer = driverContainer;
    this.expectedElementSelector = expectedElementSelector;
    this.friendlyName = friendlyName;
    this.url = url;
  }
  
  initialize() {
    let _this = this;
    return _this.driverContainer.driver.get(_this.url).then(function() {
      return _this.waitForPage();
    });
  }
  
  rest(value = null){
    return this.driverContainer.driverUtilities.rest(value);
  }
  
  scrollToTop() {
    return this.driverContainer.driverUtilities.scrollToTop();
  }
  
  waitForPage() {
    return this.driverContainer.driverUtilities.waitTillPresentAndDisplayed(this.expectedElementSelector);
  }
  
  displayed() {
    return this.driverContainer.driverUtilities.isEventuallyPresentAndDisplayed(this.expectedElementSelector);
  }
  
  title() {
    return this.driverContainer.driver.getTitle();
  }
  
  urlDisplayed() {
    return this.driverContainer.driver.getCurrentUrl();
  }
  
  checkForConsoleErrors() {
    return this.driverContainer.driverUtilities.checkForConsoleErrors();
  }
  
  checkWindowInEyes(step = "") {
    let name = this.friendlyName;
    if (step) name += " - " + step;
    return this.driverContainer.eyes.checkWindow(name);
  }
  
  takeScreenShotToFile() {
    let _this = this;
    if (_this.driverContainer.config.saveAllScreenshots === true) {
      const prefix = (!_this.friendlyName) ?
        `BaseComponent-${_this.driverContainer.config.browserType}` :
        `${_this.friendlyName}-${_this.driverContainer.config.browserType}`;
      try {
        return _this.driverContainer.driver.takeScreenshot().then((data) => {
          MediaUtilities.writeScreenshot(data, prefix);
        } );
      } catch ( e ) {
        console.log( `Error when taking screenshot in BaseComponent: '${e}'` );
      }
    }
  }
  
}
