'use strict';

import { forEach } from 'lodash';
import webdriver from 'selenium-webdriver';

class DriverUtilities {
  constructor(config, logger, driver, timeout) {
    this.config = config;
    this.logger = logger;
    this.driver = driver;
    this.defaultTimeout = timeout;
  }
  
  // Refactor 1
  rest(value = null) {
    let _this = this;
    let restValue = (value) || _this.config.sleepConstant;
    return _this.driver.sleep(restValue);
  }
  
  // Refactor 1
  scrollToElementObject(element, top = false) {
    let _this = this;
    if (top) {
      return _this.rest().then(function () {
        return _this.driver.executeScript("arguments[0].scrollIntoView(true);", element);
      }).then(function () {
        return _this.rest();
      }).then(function () {
        return _this.driver.executeScript("window.scrollBy(0,"+_this.config.scrollBuffer+")");
      });
    }
    else {
      return _this.rest().then(function () {
        return _this.driver.executeScript("arguments[0].scrollIntoView(false);", element);
      }).then(function () {
        return _this.rest();
      }).then(function () {
        return _this.driver.executeScript("window.scrollBy(0,"+_this.config.scrollBuffer+")");
      });
    }
  }
  
  // Refactor 1
  clickElementObject(element) {
    let _this = this;
    return _this.rest().then(function () {
      return element.click();
    });
  }
  
  // Refactor 1
  scrollToElement(selector, top = false) {
    let _this = this;
    return _this.driver.findElement(selector).then(function(element) {
      return _this.scrollToElementObject(element, top);
    }, function () {
      return Promise.resolve();
    });
  }
  
  // Refactor 1
  scrollToTop() {
    let _this = this;
    return _this.rest().then(function () {
      return _this.driver.executeScript("window.scrollTo(0, 0);");
    }).then(function () {
      return _this.rest();
    });
  }
  
  // Refactor 1
  _scrollToAndClickElementObject(element) {
    let _this = this;
    return _this.scrollToElementObject(element).then(function () {
      return _this.clickElementObject(element);
    });
  }
  
  // Refactor 1
  _scrollToAndClickElementObjects(elements) {
    let _this = this;
    if (elements.length === 0) return Promise.resolve();
    return _this._scrollToAndClickElementObject(elements[0]).then(function () {
      return _this.rest();
    }).then(function () {
      elements.shift();
      return _this._scrollToAndClickElementObjects(elements);
    });
  }
  
  // Refactor 1
  clickAllWhenClickable(selector) {
    let _this = this;
    return _this.driver.wait(function() {
      return _this.driver.findElements(selector).then(function(elements) {
        return _this._scrollToAndClickElementObjects(elements);
      }, function () {
        // None found, but thats okay
        return Promise.resolve();
      }).then(function () {
        return true;
      });
    }, _this.defaultTimeout, `Timed out waiting for all elements with ${selector.using} of '${selector.value}' to be clickable`);
  }
  
  // Refactor 1
  clickWhenClickable(selector) {
    let _this = this;
    return _this.driver.wait(function() {
      return _this.driver.findElement(selector).then(function(element) {
        return _this._scrollToAndClickElementObject(element);
      }).then(function () {
        return true;
      });
    }, _this.defaultTimeout, `Timed out waiting for element with ${selector.using} of '${selector.value}' to be clickable`);
  }
  
  // Refactor 1
  clickIfPresent(selector) {
    let _this = this;
    return _this.driver.findElement(selector).then(function(element) {
      return _this.clickElementObject(element);
    }, function () {
      return Promise.resolve();
    });
  }
  
  // Refactor 1
  scrollToAndClickIfPresent(selector) {
    let _this = this;
    return _this.driver.findElement(selector).then(function(element) {
      return _this._scrollToAndClickElementObject(element);
    }, function () {
      return Promise.resolve();
    });
  }

  
  
  
  // Old needs to be refactored I guess...
  
  followLinkWhenFollowable(selector) {
    let _this = this;
    return _this.driver.wait(function() {
      return _this.driver.findElement(selector).then(function(element) {
        return element.getAttribute('href').then(function(href) {
          _this.driver.get(href);
          return true;
        },function() {
          return false;
        });
      },function() {
        return false;
      });
    }, _this.defaultTimeout, `Timed out waiting for link with ${selector.using} of '${selector.value}' to be followable`);
  }
  
  waitTillPresentAndDisplayed(selector) {
    let _this = this;
    return _this.driver.wait(function() {
      return _this.driver.findElement(selector).then(function(element) {
        return element.isDisplayed().then(function() {
          return true;
        },function() {
          return false;
        });
      },function() {
        return false;
      });
    }, _this.defaultTimeout, `Timed out waiting for element with ${selector.using} of '${selector.value}' to be present and displayed`);
  }
  
  isEventuallyPresentAndDisplayed(selector) {
    let _this = this;
    return _this.driver.wait(function() {
      return _this.driver.findElement(selector).then(function(element) {
        return element.isDisplayed().then(function() {
          return true;
        },function() {
          return false;
        });
      },function() {
        return false;
      });
    }, _this.defaultTimeout).then(function(shown) {
      return shown;
    }, function() {
      return false
    });
  }
  
  waitForFieldClearable(selector) {
    let _this = this;
    return _this.driver.wait(function() {
      return _this.driver.findElement(selector).then(function(element) {
        return element.clear().then(function() {
          return element.getAttribute('value').then(function(value) {
            return value === '';
          });
        },function() {
          return false;
        });
      },function() {
        return false;
      });
    }, _this.defaultTimeout, `Timed out waiting for element with ${selector.using} of '${selector.value}' to be clearable`);
  }
  
  setWhenSettable(selector, value, secureValue = false) {
    let _this = this;
    const logValue = (secureValue === true) ? '*********' : value;
    return _this.driver.wait(function() {
      return _this.driver.findElement(selector).then(function(element) {
        _this.waitForFieldClearable(selector);
        return element.sendKeys(value).then(function() {
          return element.getAttribute('value').then(function(actualValue) {
            return actualValue === value;
          });
        },function() {
          return false;
        });
      },function() {
        return false;
      });
    }, _this.defaultTimeout, `Timed out waiting for element with ${selector.using} of '${selector.value}' to be settable to: '${logValue}'`);
  }
  
  setCheckbox(selector) {
    let _this = this;
    return _this.driver.findElement(selector).then(function(checkbox) {
      checkbox.getAttribute('checked').then(function(checked) {
        if (checked !== 'true') {
          return _this.clickWhenClickable(selector);
        }
      });
    });
  }
  
  unsetCheckbox(selector) {
    let _this = this;
    return _this.driver.findElement(selector).then(function(checkbox) {
      checkbox.getAttribute('checked').then(function(checked) {
        if (checked === 'true') {
          return _this.clickWhenClickable(selector);
        }
      });
    });
  }
  
  /**
   * Check whether an image is actually visible - that is rendered to the screen - not just having a reference in the DOM
   * @param {WebElement} webElement - Element to search for
   * @returns {Promise} - Resolved when the script is done executing
   */
  imageVisible(webElement) {
    let _this = this;
    return _this.driver.executeScript('return (typeof arguments[0].naturalWidth!=\"undefined\" && arguments[0].naturalWidth>0)', webElement);
  }
  
  checkForConsoleErrors() {
    let _this = this;
    if (_this.config.checkForConsoleErrors === true) {
      _this.driver.manage().logs().get('browser').then(function(logs) {
        if (logs.length > 0) {
          forEach(logs, function(log) {
            // Ignore chrome cast errors in Chrome - http://stackoverflow.com/questions/24490323/google-chrome-cast-sender-error-if-chrome-cast-extension-is-not-installed-or-usi/26095117#26095117
            if (log.message.indexOf('cast_sender.js') === -1) {
              _this.driver.getCurrentUrl().then(function(url) {
                _this.logger.warn(`Found console error: "${log.message}" on url '${url}'`, true);
              });
            }
          });
        }
      });
    }
  }
}
module.exports.DriverUtilities = DriverUtilities;
